import serial
from thread import start_new_thread, allocate_lock
import time

class Display:
	def __init__(self,portname="/dev/ttyUSB0"):
		self.serialport=None
		self.portlock = allocate_lock()
		self.portname=portname
		self.open_port()
		self.brightness=0
		self.screensaver = Screensaver(self)
		self.screensaver.start()

	def __del__(self):
		if (not self.screensaver == None):
			self.screensaver.terminate()
			self.screensaver = None

		if (not self.serialport == None):
			self.serialport.close()
			self.serialport=None

	def open_port(self):

		self.mutex_get()
		try:
			self.serialport = serial.Serial(self.portname,9600,timeout=2,rtscts=True, dsrdtr=True)
			print ("Initializing display on port:\n  %s\n" % self.serialport )
			self.cmd_reset()
			self.mutex_release()
		except Exception as e:
			print ("Exception opening serial port '%s' for display\n" % self.portname)
			print ("Ignoring and trying to open it again later\n")
			print (e)
			self.serialport = None
			self.mutex_release()
			pass
		



	def cmd_reset(self):
		#reset the display
		self.cmd("\x1b\x40")

	def cmd_cursor_show(self,on=True):
		#show or hide the cursor
		if (on): 
			self.cmd("\x1f\x43\x01")
		else:
			self.cmd("\x1f\x43\x00")

	def cmd_blink(self,rate=127):
		self.cmd("\x1f\x45%c" % chr(rate))


	def cmd_brightness(self,level):
		if (level==0): #turn off:
			self.cmd_display_on(False)	
                        self.brightness = level
                        return
		else:  
			if (self.brightness==0): #turn on, then set wanted brightness:
				self.cmd_display_on(True)	
                        self.brightness = level
                        self.cmd("\x1F\x58%c" % chr(level-1) )

	def cmd_display_on(self,on=True):
		# blink-command: 
		# where the last value defines the blink rate, 
		# with 0=always on, 255=always off
		if (on):
			self.cmd_blink(0)
		else:
			self.cmd_blink(255)

	def cmd_curmode_overwrite(self):
		cmd_curmode(1)			

	def cmd_curmode_vscroll(self):
		cmd_curmode(2)			

	def cmd_curmode_hscroll(self):
		cmd_curmode(3)			

	def cmd_curmode(self,val=2):
		self.cmd("\x1f%c" % chr(val))

	def cmd_clear(self):
		self.cmd("\x0c")

	
	def cmd_time(self):
		# activate timer display in lower right corner.
		# actual time has to be set via display_set_timer.
		self.cmd("\x1f\x55")

	def cmd_time_set(self,h=None,m=None):
		if (h==None or m==None):
			now = time.localtime()
			h=now.tm_hour
			m=now.tm_min
		# configures hour and minutes of the internal clock and
		# displays the time in the lower right corner.
		# turns off when something gets written into the line.
		# two bytes have to follow with hour and minute
		self.cmd("\x1F\x54%c%c" % (chr(h),chr(m)) )
		self.cmd_curmode_hscroll() # makes the time more robust...

	def cmd(self,text): #for commands: do not reset screensaver timeout
		self.write(text,False)

	def write(self,text,idle_reset=True): #for data: reset screensaver timeout
		if (not self.serialport == None):
			if (idle_reset):
				self.screensaver.idle_reset();
			self.serialport.write(text)	
		else:
			self.open_port()	


	def mutex_get(self):
		self.portlock.acquire()

	def mutex_release(self):
		self.portlock.release()

	def terminate(self):
		if (not self.screensaver == None):
			self.screensaver.terminate()
			self.screensaver = None
		

import threading
import time

class Screensaver (threading.Thread):
	def __init__(self,display):
		self.display=display
		self.runme=True
		self.idlecounter=0
		self.timeout_dim=30
		self.timeout_message=60
		self.timeout_off=120
		self.mutex = allocate_lock()
	        threading.Thread.__init__(self)

	def run(self):
		print "Starting Screensaver thread" 
		while(self.runme):
			self.mutex.acquire()
			self._main_loop()
			self.mutex.release()
			time.sleep(.5)
		print "Exiting Screensaver thread"

	def terminate(self):
		self.runme = False


	def idle_reset(self):
		self.mutex.acquire()
		self.idlecounter=0
		self.mutex.release()

	def _main_loop(self):
		if (self.idlecounter < self.timeout_off):
			self.idlecounter+=1

		if (self.idlecounter>=self.timeout_dim and self.idlecounter <= (self.timeout_dim+7)):
			x = (8-( self.idlecounter - self.timeout_dim))/2
			self.display.mutax_get()
			self.display.cmd_brightness(1+x)
			self.display.mutax_release()
			return

		if (self.idlecounter==self.timeout_message):
			return

		if (self.idlecounter==self.timeout_off):
			self.display.mutax_get()
                        display.cmd_display_on(False)
			self.display.mutax_release()
			return
		
	def __del__(self):
		print("bye")

